use clap::{App, Arg};
use pdbget;
use reqwest;
use std::{fs, io, io::Write, path::PathBuf};

fn handled_reqwest_errors(uri: &str, e: reqwest::Error) -> io::Error {
    io::Error::new(
        io::ErrorKind::ConnectionAborted,
        format!("[Warning] when connecting to {}\n{}", uri, e.to_string()),
    )
}

fn get(uri: &str, writer: &mut Vec<u8>) -> Result<(), io::Error> {
    let mut result = match reqwest::Client::new().get(uri).send() {
        Ok(res) => res,
        Err(e) => return Err(handled_reqwest_errors(uri, e)),
    };
    if result.status() != 200 {
        return Err(io::Error::new(
            io::ErrorKind::ConnectionAborted,
            format!(
                "[Warning] when connecting to {}\nResponse Statut : {}",
                uri,
                result.status()
            ),
        ));
    }
    match result.copy_to(writer) {
        Ok(_) => Ok(()),
        Err(e) => Err(handled_reqwest_errors(uri, e)),
    }
}

fn write_file(filename: PathBuf, writer: Vec<u8>) -> Result<(), io::Error> {
    let mut output_pdb = fs::File::create(filename)?;
    output_pdb.write(&writer)?;

    Ok(())
}

/// Check the output directory pass in argument of the program
/// Check if the folder exist, if not create it
/// Return an error if it's not possible, or in case of wrong permissions access
fn check_output(output: &str) -> Result<PathBuf, io::Error> {
    let path = PathBuf::from(output);
    if path.is_dir() {
        return Ok(path);
    }
    fs::create_dir(&path)?;
    Ok(path)
}

fn main() -> Result<(), io::Error> {
    let matches = App::new("Get Protein files")
        .version("0.1.0")
        .author("Philippe Noel")
        .about("Download Protein files on rcsb.org")
        .arg(
            Arg::with_name("PDBs")
                .help("PDB identifiers")
                .required(true)
                .min_values(1),
        )
        .arg(
            Arg::with_name("Output")
                .help("Output folder where to store files")
                .short("o")
                .default_value("./"),
        )
        .arg(
            Arg::with_name("UriType")
                .help("File type to download. 'PDB', 'CIF', 'FASTA'")
                .short("t")
                .default_value("PDB"),
        )
        .get_matches();

    let pdb_identifiers: Vec<_> = matches.values_of("PDBs").unwrap().collect(); // Call unwrap because PDBs is required in clap
    let output_folder = check_output(matches.value_of("Output").unwrap())?; // Use unwrap because default is set

    let uri_type = pdbget::uri_types::UriType::get_type(matches.value_of("UriType").unwrap())?; // Use unwrap because default is set
    let extension = uri_type.get_extension();

    for pdb in pdb_identifiers {
        let mut writer: Vec<u8> = vec![]; // Create a buffer writer which will contain each bytes of the GET request
        match get(&uri_type.generate_uri(pdb), &mut writer) {
            Ok(_) => (),
            Err(e) => {
                eprintln!("[Warning] problem with '{}' id\n{}", pdb, e);
                continue; // Got to the next iteration
            }
        };
        let mut output_filename = output_folder.clone();
        output_filename.push(format!("{}{}", pdb, extension));
        write_file(output_filename, writer)?;
    }

    Ok(())
}
