use std::io;

pub enum UriType {
    Fasta(String),
    Pdb(String),
    PdbGz(String),
    Cif(String),
    CifGz(String),
    XmlGz(String),
}

impl UriType {
    pub fn get_type(t: &str) -> Result<UriType, io::Error> {
        let t = String::from(t);
        match t.to_uppercase().as_ref() {
            "FASTA" => Ok(UriType::Fasta(t)),
            "PDB" => Ok(UriType::Pdb(t)),
            "PDBGZ" => Ok(UriType::PdbGz(t)),
            "CIF" => Ok(UriType::Cif(t)),
            "CIFGZ" => Ok(UriType::CifGz(t)),
            "XML" => Ok(UriType::XmlGz(t)),
            _ => Err(io::Error::new(
                io::ErrorKind::InvalidInput,
                format!(
                    "[Error] '{}' is not an accepted format. Use -h to see available types",
                    t
                ),
            )),
        }
    }

    pub fn generate_uri(&self, id: &str) -> String {
        match self {
            UriType::Fasta(_) => format!("https://www.rcsb.org/pdb/download/downloadFastaFiles.do?structureIdList={}&compressionType=uncompressed", id),
            UriType::Pdb(_) => format!("https://files.rcsb.org/download/{}.pdb", id),
            UriType::PdbGz(_) => format!("https://files.rcsb.org/download/{}.pdb.gz", id),
            UriType::Cif(_) => format!("https://files.rcsb.org/download/{}.cif", id),
            UriType::CifGz(_) => format!("https://files.rcsb.org/download/{}.cif.gz", id),
            UriType::XmlGz(_) => format!("https://files.rcsb.org/download/{}.xml.gz", id),
        }
    }

    pub fn get_extension(&self) -> &str {
        match self {
            UriType::Fasta(_) => ".fasta",
            UriType::Pdb(_) => ".pdb",
            UriType::Cif(_) => ".cif",
            UriType::PdbGz(_) => ".pdb.gz",
            UriType::CifGz(_) => ".cif.gz",
            UriType::XmlGz(_) => ".xml.gz",
        }
    }
}
